
// Copyright (c) BigFredi 2004.
// Author: Alfredo Garca Martino
// All rights reserved.

#include "./rtbapi/include/robot.hpp"
#include "./rtbapi/include/consts.hpp"
#include <string>
#include <iostream>
#include <cstdlib>
#define PI 3.1416
#define NO_GIRA 0
#define DERECHA 1
#define IZQUIERDA 2
#define MENOR_ANGULO_DE_SWEEP 0.10
#define MAYOR_ANGULO_DE_SWEEP 0.50
#define TIEMPO_MAXIMO_SIN_VER_GALLETA 3
#define TIEMPO_MAXIMO_SIN_VER_PRESA 4
#define FACTOR_AJUSTE_VELOCIDAD 100
#define FACTOR_AJUSTE_DISTANCIA 10


const std::string robot_name = "DeathBlow";
const int color_1 = 0xcc0000;
const int color_2 = 0x006688;
const int MAX_VEL = 5;

class EuskalRobot : public rtb::Robot {
  struct {
    double distance;
    rtb::Consts::object_type object;
    double radar_angle;
    double time;
    double speed;
    double cannon_angle;
    double energy;
    int teammate;
  } info;

  double angulo_de_sweep;
  int direccion_de_giro;
  int rotaciones;
  int primera_rotacion;
  int valor_de_rotacion;
  int rotando;
  int cazando;
  int colocandome;
  int a_por_galletas;
  int choque_inminente;
  int velocidad_maxima;
  int prueba_de_rotacion;
  int tiempo_sin_ver_galleta;
  int galleta_enfocada;
  int tiempo_sin_ver_presa;
  int presa_enfocada;
  int tiempo_de_disparo;
  int potencia_de_disparo;
  double distancia_a_presa_anterior;
  double distancia_a_presa_actual;
  double diferencial_de_distancia;
  void actua ();

public:
  EuskalRobot (std::string name, int colour, int away_colour);
  virtual ~EuskalRobot () { }
  void GameStarts ();
  void Radar (double distance, rtb::Consts::object_type object, double radar_angle);
  void Info (double time, double speed, double cannon_angle);
  void RotationReached (rtb::Consts::rotate_type what);
  void Collision (rtb::Consts::object_type object, double angle);
  void RobotInfo (double energy, int teammate);
  void Rotar();
  void Aumentar_Angulo_De_Sweep();
  void Reducir_Angulo_De_Sweep();
  void Donde ();

};

EuskalRobot::EuskalRobot (std::string name, int colour, int away_colour): Robot::Robot (name, colour, away_colour) {
}

void EuskalRobot::GameStarts () {
  rotando = 0;
  primera_rotacion=0;
  valor_de_rotacion=0;
  choque_inminente = 0;
  colocandome=0;
  angulo_de_sweep = 0.5;
  a_por_galletas = 0;
  tiempo_sin_ver_galleta=0;
  galleta_enfocada = 0;
  cazando = 0;
  tiempo_sin_ver_presa=0;
  presa_enfocada = 0;
  velocidad_maxima=MAX_VEL;
  direccion_de_giro=DERECHA;
  prueba_de_rotacion=1;
  tiempo_de_disparo=0;
  potencia_de_disparo=0;
  Sweep (rtb::Consts::ROTATE_CANNON_RADAR, 1000, -angulo_de_sweep, angulo_de_sweep);
  Accelerate(2000);
}

void EuskalRobot::Radar (double distance, rtb::Consts::object_type object, double radar_angle) {
  info.distance = distance;
  info.object = object;
  info.radar_angle = radar_angle;
}

// Recibimos informacin:
void EuskalRobot::Info (double time, double speed, double cannon_angle) {
  info.time = time;
  info.speed = speed;
  info.cannon_angle = cannon_angle;
  // Si el objeto no es un robot:
  actua ();
}

//Para calcular donde est el objetivo
void EuskalRobot::Donde () {
	double angulo_temporal;
	if ((rotando == 0) && (colocandome == 0)) {
		colocandome=1;
//		Calculemos el giro que vamos a realizar:
//		La base es colocarnos mirando hacia el objeto:
		angulo_temporal=angulo_de_sweep/2;
//		Lo ajustamos con respecto a la distancia: < distancia ms angulo
//		angulo_temporal=angulo_temporal+((angulo_temporal/FACTOR_AJUSTE_DISTANCIA)/info.distance);
//		Lo ajustamos con respecto a la velocidad: > velocidad ms angulo
		angulo_temporal=angulo_temporal+((angulo_temporal/FACTOR_AJUSTE_VELOCIDAD)*info.speed);
		if (direccion_de_giro==DERECHA) {
			RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, info.radar_angle-angulo_temporal);
		} else {
			RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, info.radar_angle+angulo_temporal);
		}
	}
}
// Parada de la rotacin:
void EuskalRobot::RotationReached (rtb::Consts::rotate_type what) {
	switch (what) {
		case rtb::Consts::ROTATE_ROBOT:
//			Print ("Rotacion de Bot finalizada");
			Accelerate(100);
			rotando = 0;
			colocandome=0;
			Brake(0);
		break;
		case rtb::Consts::ROTATE_CANNON:
//			Print ("Rotacion de caon finalizada");
		break;
		case rtb::Consts::ROTATE_ROBOT_CANNON:
//			Print ("Rotacion de rob-ca finalizada");
		break;
		case rtb::Consts::ROTATE_RADAR:
//			Print ("Rotacion de radar finalizada");
		break;
		case rtb::Consts::ROTATE_ROBOT_RADAR:
//			Print ("Rotacion de rob-rad finalizada");
		break;
		case rtb::Consts::ROTATE_CANNON_RADAR:
//			Print ("Rotacion de ca-rad finalizada");
			if (prueba_de_rotacion)
			{
				if (info.radar_angle <0)
				{
					direccion_de_giro=DERECHA;
				} else {
					direccion_de_giro=IZQUIERDA;
				}
				prueba_de_rotacion=0;
			}
			if (direccion_de_giro == DERECHA) {
				direccion_de_giro=IZQUIERDA;
//				Print ("A la Izquierda");
			} else {
				direccion_de_giro=DERECHA;
//				Print ("A la Derecha");
			}
		break;
		case rtb::Consts::ROTATE_ALL:
//			Print ("Rotacion de todo finalizada");
			Accelerate(100);
			rotando = 0;
			colocandome=0;
			Brake(0);
		break;
		case rtb::Consts::ROTATE_CAGADA:
//			Print ("Rotacion cagada");
			Accelerate(1000);
			rotando = 0;
		break;
		default:
//			Print ("Rotacion desconocido finalizada");
			rotando = 0;
		break;
	}
}


// Si chocamos:
void EuskalRobot::Collision (rtb::Consts::object_type object, double angle) {
// 	Hemos chocado con la pared:
	if ((object == rtb::Consts::IS_SHOT) && (rotando == 0)) {
		RotateAmount(rtb::Consts::ROTATE_ROBOT,1000,angle);
		rotando=1;
	}
}

void EuskalRobot::RobotInfo (double energy, int teammate) {
  info.energy = energy;
  info.teammate = teammate;
  actua ();
}

void EuskalRobot::actua () {
//	Control de velocidad:
	if (info.speed < velocidad_maxima) {
		Accelerate (1000);
	} else {
		Accelerate (-1000);
	}
	if (info.object!=rtb::Consts::IS_COOKIE) {
//		Print ("No veo galletas");
		if (a_por_galletas) {
			tiempo_sin_ver_galleta++;
			if (tiempo_sin_ver_galleta>=TIEMPO_MAXIMO_SIN_VER_GALLETA) {
				a_por_galletas = 0;
				Print ("He perdido la galleta");
			}
		}
		galleta_enfocada = 0;
	}
	if (info.object!=rtb::Consts::IS_ROBOT) {
//		Print ("No veo robots");
		if (cazando)
		{
			tiempo_sin_ver_presa++;
			if (tiempo_sin_ver_presa>=TIEMPO_MAXIMO_SIN_VER_PRESA) {
				cazando = 0;
				Print ("He perdido la Presa");
			}
		}
		presa_enfocada = 0;
	}
	if (tiempo_de_disparo>0) {
		if (potencia_de_disparo>10){
			if (angulo_de_sweep<=MAYOR_ANGULO_DE_SWEEP/2) {
				potencia_de_disparo=20;
			} else {
				potencia_de_disparo=5;
			}
		}
		Shoot(potencia_de_disparo);
		tiempo_de_disparo--;
	}
//	Accin dependiente del resultado del radar:
	switch (info.object) {
//		Es un robot:
		case rtb::Consts::IS_ROBOT:
			cazando = 1;
			// Activar modo caza
			if (!rotando) {
				if (!presa_enfocada) {
					Print ("Nos vamos de caza");
					cazando = 1;
					rotaciones = 0;
					if (!colocandome)
						Reducir_Angulo_De_Sweep();
					Donde();
				}
			} else {
				rotaciones=0;
			}
			
			presa_enfocada = 1;			
			if (info.distance <= 1){
				potencia_de_disparo=50;
				tiempo_de_disparo=2;
				velocidad_maxima=MAX_VEL;				
			} else if (info.distance <= 2) {
				potencia_de_disparo=25;
				tiempo_de_disparo=3;
				velocidad_maxima=MAX_VEL/5;
			} else if (info.distance <= 3) {
				potencia_de_disparo=10;
				tiempo_de_disparo=4;
				velocidad_maxima=MAX_VEL/4;
			} else if (info.distance <= 5) {
				potencia_de_disparo=5;
				tiempo_de_disparo=5;
				velocidad_maxima=MAX_VEL/3;
			} else {
				tiempo_de_disparo=5;
				potencia_de_disparo=1;
				velocidad_maxima=MAX_VEL;
			}
			Shoot (potencia_de_disparo);

		break;
//		Es una cookie:
		case rtb::Consts::IS_COOKIE:
			if ((!rotando) && (!cazando) && (!galleta_enfocada)) {
//				Print ("A por galletas");
				a_por_galletas = 1;
				tiempo_sin_ver_galleta = 0;
				rotaciones = 0;
				Donde ();
			}
//			Print ("Veo galleta");
			galleta_enfocada = 1;
		break;
//		Es una mina:
		case rtb::Consts::IS_MINE:
//			Print ("Mina ...");
			Shoot (1);
		break;
//		Es un muro:
		case rtb::Consts::IS_WALL:
			if ((!cazando) && (!a_por_galletas)) {
				switch ((int)info.distance) {
					case 5: 
						velocidad_maxima=3*MAX_VEL/4;
					break;
					case 4:
						velocidad_maxima=MAX_VEL/2;
					break;
					case 3:
						velocidad_maxima=MAX_VEL/3;
					break;
					case 2:
						velocidad_maxima=MAX_VEL/4;
					break;
					case 1:
						velocidad_maxima=MAX_VEL/5;
					break;
					case 0:
					break;
					default:
						velocidad_maxima=MAX_VEL;
				}
				if ((!rotando) && (rotaciones==0) && ((int)info.distance<=5)) {
					primera_rotacion=1;
//					rotaciones=(int)(6-info.distance);
					if ((int)info.distance==5){
						rotaciones=1;
					} else if (((int)info.distance<5)&&((int)info.distance>2)){
						rotaciones=2;
					} else {
						rotaciones=3;
					}
				}
				if (!rotando) {
					Rotar ();
				}
			}
		break;
//		Es un objeto desconocido:
		case rtb::Consts::IS_NOOBJECT:
			rotaciones=2;
			Rotar();
		break;
	}
}

void EuskalRobot::Aumentar_Angulo_De_Sweep(){
	if (angulo_de_sweep<=MAYOR_ANGULO_DE_SWEEP) {
		angulo_de_sweep=angulo_de_sweep-(angulo_de_sweep/5);
	}
}

void EuskalRobot::Reducir_Angulo_De_Sweep(){
	if (angulo_de_sweep>=MENOR_ANGULO_DE_SWEEP) {
		angulo_de_sweep=angulo_de_sweep-(angulo_de_sweep/5);
	}
}
void EuskalRobot::Rotar() {
	if (rotaciones>0) {
		if (primera_rotacion) {
			if (info.radar_angle < 0) {
				valor_de_rotacion=1;
			} else {
				valor_de_rotacion=-1;
			}
			primera_rotacion=0;
		}
		RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, valor_de_rotacion*3.1416/4);
		rotando = 1;
		Brake(1);
		rotaciones --;
	}
}
// Procedimiento main:
int main(void) {
  EuskalRobot sample (robot_name, color_1, color_2);

  sample.Scanner();

  return 0;
}
