
// Copyright (c) Jacobo Tarro Barreiro.
// All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither Jacobo Tarro Barreiro's name nor the names of his contributors
//    may be used to endorse or promote products derived from this software
//    without specific prior written permission.

// THIS SOFTWARE IS PROVIDED BY JACOBO TARRO BARREIRO AND THE CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JACOBO TARRO BARREIRO OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "robot.hpp"
#include "consts.hpp"
#include <string>
#include <cstdlib>

const std::string robot_name = "rtbapi-- example";
const int color_1 = 0xf01122;
const int color_2 = 0xfdb975;

class SampleRobot : public rtb::Robot {
  struct {
    double distance;
    rtb::Consts::object_type object;
    double radar_angle;
    double time;
    double speed;
    double cannon_angle;
    double energy;
    int teammate;
  } info;

  int refract;

  void actua ();

public:
  SampleRobot (std::string name, int colour, int away_colour);
  virtual ~SampleRobot () { }
  void GameStarts ();
  void Radar (double distance, rtb::Consts::object_type object, double radar_angle);
  void Info (double time, double speed, double cannon_angle);
  void Collision (rtb::Consts::object_type object, double angle);
  void RobotInfo (double energy, int teammate);

};

SampleRobot::SampleRobot (std::string name, int colour, int away_colour)
  : Robot::Robot (name, colour, away_colour) {
}

void SampleRobot::GameStarts () {
  Accelerate (1);
  Sweep (rtb::Consts::ROTATE_CANNON_RADAR, 1000, -0.5, 0.5);
  refract = 0;
}

void SampleRobot::Radar (double distance, rtb::Consts::object_type object, double radar_angle) {
  info.distance = distance;
  info.object = object;
  info.radar_angle = radar_angle;
}

void SampleRobot::Info (double time, double speed, double cannon_angle) {
  info.time = time;
  info.speed = speed;
  info.cannon_angle = cannon_angle;

  if (info.object != rtb::Consts::IS_ROBOT) {
    actua ();
  }
}

void SampleRobot::Collision (rtb::Consts::object_type object, double angle) {
  if (object == rtb::Consts::IS_SHOT) {
    if ((angle < -0.5) || (angle > 0.5)) {
      Print ("Ouch!");
      Accelerate (1000);
    }
  }
}

void SampleRobot::RobotInfo (double energy, int teammate) {
  info.energy = energy;
  info.teammate = teammate;

  if (!info.teammate) {
    actua ();
  }
}

void SampleRobot::actua () {
  if (refract)
    refract--;
  switch (info.object) {
  case rtb::Consts::IS_ROBOT:
    RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, info.radar_angle);
    if (info.distance <= 1)
      Shoot (100);
    else if (info.distance <= 2)
      Shoot (5);
    else if (info.distance <= 3)
      Shoot (3);
    else if (info.distance <= 5)
      Shoot (2);
    else
      Shoot (1);
    Accelerate (0.45);
    break;
  case rtb::Consts::IS_COOKIE:
    RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, info.radar_angle);
    Accelerate (5);
    break;
  case rtb::Consts::IS_MINE:
    Shoot (0.5);
    break;
  case rtb::Consts::IS_WALL:
    if (info.distance <= 4) {
      Accelerate (-10);
      if ((refract == 0) && (info.radar_angle < 0)) {
	RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, 3.1416);
	refract = 250;
      } else {
	RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, -3.1416);
	refract = 250;
      }
      break;
    }
  case rtb::Consts::IS_NOOBJECT:
    Accelerate (((random() % 40) - 10) / 10);
    RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, ((random() % 40) - 20) / 10);
  }
}

int main(void) {
  SampleRobot sample (robot_name, color_1, color_2);

  sample.Scanner();

  return 0;
}
