
// Copyright (c) Jacobo Tarro Barreiro.
// All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither Jacobo Tarro Barreiro's name nor the names of his contributors
//    may be used to endorse or promote products derived from this software
//    without specific prior written permission.

// THIS SOFTWARE IS PROVIDED BY JACOBO TARRO BARREIRO AND THE CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JACOBO TARRO BARREIRO OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <robot.hpp>
#include <cstdlib>

class SRobot : public rtb::Robot {
  bool stop;
public:
  SRobot (std::string name, int colour, int colour2);
  void GameStarts ();
  void Radar (double distance, rtb::Consts::object_type object, double angle);
  void Info (double time, double speed, double cannon_angle);
  void Collision (rtb::Consts::object_type object, double angle);
  void RotationReached (rtb::Consts::rotate_type what);
};

SRobot::SRobot (std::string name, int colour, int colour2)
  : Robot (name, colour, colour2) { stop = false; }

void SRobot::GameStarts () {
  Sweep (rtb::Consts::ROTATE_CANNON_RADAR, 1000, -0.5, 0.5);
}

void SRobot::Radar (double distance, rtb::Consts::object_type object, double angle) {
  if (object == rtb::Consts::IS_ROBOT) {
    Shoot (1.5);
    RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, angle);
  }
}

void SRobot::Info (double time, double speed, double cannon_angle) {
  Accelerate (2);
}

void SRobot::Collision (rtb::Consts::object_type object, double angle) {
  if (!stop) {
    RotateAmount (rtb::Consts::ROTATE_ROBOT, 1000, ((random() % 62832) - 31416) / 10000);
    stop = true;
  }
}

void SRobot::RotationReached (rtb::Consts::rotate_type what) {
  stop = false;
}

int main(int argc, char *argv[]) {
  SRobot shooter ("Shooter", 0x73c3d7, 0x2ed322);

  shooter.Scanner ();

  return 0;
}
